// Character type class -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "CharType.h"
#include "LeafValue.h"
#include "Constraint.h"
#include "Range.h"

/** @file CharType.C
 * Character data type
 */

class Value&
CharType::getFirstValue () const
{
  const class Value* v = myConstraint ? &myConstraint->getFirstValue () : NULL;
  assert (!v || &v->getType () == this);
  return v ? *v->copy () : *new class LeafValue (*this, 0);
}

class Value&
CharType::getLastValue () const
{
  const class Value* v = myConstraint ? &myConstraint->getLastValue () : NULL;
  assert (!v || &v->getType () == this);
  return v ? *v->copy () : *new class LeafValue (*this, CHAR_T_MAX);
}

card_t
CharType::do_getNumValues () const
{
  assert (!myConstraint);
  return CHAR_T_MAX + 1;
}

card_t
CharType::convert (const class Value& value) const
{
  assert (value.getKind () == Value::vLeaf);
  if (myConstraint)
    return Type::convert (value);
  return char_t (static_cast<const class LeafValue&>(value));
}

class Value*
CharType::convert (card_t number) const
{
  assert (number < getNumValues ());
  if (myConstraint)
    return Type::convert (number);
  return new class LeafValue (*this, number);
}

#ifdef EXPR_COMPILE
# include "StringBuffer.h"

void
CharType::compileDefinition (class StringBuffer& out,
			     unsigned indent) const
{
  out.indent (indent);
  out.append ("unsigned char");
}

void
CharType::do_compileConversion (class StringBuffer& out,
				unsigned indent,
				const char* value,
				const char* number,
				bool add) const
{
  out.indent (indent);
  out.append (number);
  out.append (add ? "+=" : "=");
  out.append (value);
  out.append (";\n");
}

#endif // EXPR_COMPILE
