// Value list class -*- c++ -*-

#ifndef VALUELIST_H_
# define VALUELIST_H_
# ifdef __GNUC__
#  pragma interface
# endif // __GNUC__

# include <cassert>
# include "typedefs.h"

/** @file ValueList.h
 * List of values (components of StructValue, BufferValue or VectorValue)
 */

/** Value list */
class ValueList
{
public:
  /** Constructor
   * @param size	Size of the value list (number of elements)
   */
  ValueList (card_t size);
  /** Copy constructor */
  ValueList (const class ValueList& old);
private:
  /** Assignment operator */
  class ValueList& operator= (const class ValueList& old);
public:
  /** Destructor */
  ~ValueList ();

  /** Determine the size of the list */
  card_t getSize () const { return mySize; }
  /** Address an element by index */
  class Value*& operator[] (card_t index) {
    assert (index < getSize ());
    return myComponents[index];
  }
  /** Address an element by index */
  const class Value* operator[] (card_t index) const {
    assert (index < getSize ());
    return myComponents[index];
  }

private:
  /** Size of the list */
  card_t mySize;
  /** The values */
  class Value** myComponents;
};

#endif // VALUELIST_H_
